/*{ Template_FreeRTOS::.c ************************************************
 
 Empty template for new c-files.
 Copy and adapt to your needs.
 
}*/

#include "stm32_ADC.h"

//{ Function definitions *************************************************

/* Prepares one ADC for later configuration
 *
 * ADCx        ADC1 .. ADC2
 * WithDMA  == TRUE: enable DMA-controller for this ADC
 */
void prepareADC(ADC_TypeDef* ADCx, BOOL WithDMA) {
    /* Enable ADC clocks */
    RCC_APB2PeriphClockCmd(RCC_APB2Periph_AFIO, ENABLE);
#if defined (STM32F10X_LD_VL) || defined (STM32F10X_MD_VL)
    RCC_ADCCLKConfig(RCC_PCLK2_Div2);  // ADCCLK = PCLK2/2
#else
    //? RCC_ADCCLKConfig(RCC_PCLK2_Div4);  // ADCCLK = PCLK2/4
    RCC_ADCCLKConfig(RCC_PCLK2_Div6);
#endif

    if (WithDMA)
        RCC_AHBPeriphClockCmd(RCC_AHBPeriph_DMA1,   ENABLE); // ToDo: check if correct for ADCx==ADC2

    if (ADCx == ADC1)
        RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC1, ENABLE);
    else
        RCC_APB2PeriphClockCmd(RCC_APB2Periph_ADC2, ENABLE);

    ADC_DeInit(ADCx);

}

//} Function definitions

/* initialize single ADC channel with/ without DMA transfer
 * ADCx        ADC1 .. ADC2
 * AdcChannel  ADC_Channel_0 .. ADC_Channel_17
 * Value       != NULL: use DMA to copy sampled value into variable
 */
void initAdcChannel(ADC_TypeDef* ADCx, u8_t AdcChannel, volatile u16_t* Value) {         // configure Analog Digital Converters

    ADC_InitTypeDef   ADC_InitStructure;
    DMA_InitTypeDef   DMA_InitStructure;

    if (Value) { // DMA channel1 configuration
        DMA_DeInit(DMA1_Channel1);
        DMA_StructInit(&DMA_InitStructure);

        DMA_InitStructure.DMA_PeripheralBaseAddr = &(ADCx->DR); //?ADC1_DR_Address;
        DMA_InitStructure.DMA_MemoryBaseAddr     = (u32_t) Value;
        DMA_InitStructure.DMA_DIR                = DMA_DIR_PeripheralSRC;
        DMA_InitStructure.DMA_BufferSize         = 1;
        DMA_InitStructure.DMA_PeripheralInc      = DMA_PeripheralInc_Disable;
        DMA_InitStructure.DMA_MemoryInc          = DMA_MemoryInc_Disable;
        DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_HalfWord;
        DMA_InitStructure.DMA_MemoryDataSize     = DMA_MemoryDataSize_HalfWord;
        DMA_InitStructure.DMA_Mode               = DMA_Mode_Circular;
        DMA_InitStructure.DMA_Priority           = DMA_Priority_High;
        DMA_InitStructure.DMA_M2M                = DMA_M2M_Disable;
        DMA_Init(DMA1_Channel1, &DMA_InitStructure);

        // Enable DMA1 channel1
        DMA_Cmd(DMA1_Channel1, ENABLE);
    }

    // ADCx configuration
    ADC_InitStructure.ADC_Mode               = ADC_Mode_Independent;
    ADC_InitStructure.ADC_ScanConvMode       = ENABLE;
    ADC_InitStructure.ADC_ContinuousConvMode = ENABLE;
    ADC_InitStructure.ADC_ExternalTrigConv   = ADC_ExternalTrigConv_None;
    ADC_InitStructure.ADC_DataAlign          = ADC_DataAlign_Right;
    ADC_InitStructure.ADC_NbrOfChannel       = 1;
    ADC_Init(ADCx, &ADC_InitStructure);
    ADC_RegularChannelConfig(ADCx, AdcChannel, 1, ADC_SampleTime_55Cycles5);

    if (Value) ADC_DMACmd(ADCx, ENABLE);         // Enable ADCx DMA
    ADC_Cmd(ADCx, ENABLE);                       // Enable ADCx
    ADC_SoftwareStartConvCmd(ADCx, ENABLE);
}

/* read value from single ADC channel
 *
 * Note: blocks until ADC has finished sampling!
 *
 * ADCx        ADC1 .. ADC2
 * AdcChannel  ADC_Channel_0 .. ADC_Channel_17
 * return      sampled value
 */
unsigned int readSingleAdcChannel(ADC_TypeDef* ADCx, unsigned char Channel) {
    // Configure channel
    ADC_RegularChannelConfig(ADCx, Channel, 1, ADC_SampleTime_55Cycles5);

    // Start the conversion
    ADC_SoftwareStartConvCmd(ADCx, ENABLE);

    // Wait until conversion completion
    while (ADC_GetFlagStatus(ADCx, ADC_FLAG_EOC) == RESET);

    // Get the conversion value
    return ADC_GetConversionValue(ADCx);
}
