#ifndef STM32_IO_H
#define STM32_IO_H

//{ commonly used defines (required by FreeRTOS and/or StdPeripheralsLibrary)
typedef unsigned char        u8_t;
typedef unsigned short int  u16_t;
typedef unsigned long       u32_t;
typedef signed char          s8_t;
typedef signed short int    s16_t;
typedef signed int          s32_t;
typedef enum { FALSE = 0, TRUE } BOOL;
typedef BOOL bool;

#ifndef NULL
#define NULL 0
#endif

typedef enum ErrorCode_E {
  ec_None,                           // 0
  ec_Malloc,                         // 1
  ec_NULL,
  ec_InvalidArgument,
  ec_IllegalTimerCfg,
  ec_IllegalPwmType1,
  ec_IllegalPwmType2,
  ec_IllegalPwmType3,
  ec_IllegalPwmChannel,
  ec_IllegalPwmConfig,
  ec_InitPort_UnknownGPIO,
  ec_ServosMove_IllegalServosConfig,
  ec_UNKNOWN
} ErrorCode_e;

//} Defines
//{ includes

#include <stdlib.h>
#include "stm32f10x.h"
//? #include "StdPeripheralsLibrary/CMSIS/CM3/CoreSupport/core_cm3.h"
#include "core_cm3.h"
#include "stm32f10x_conf.h"
#include "stm32f10x_gpio.h"

// FreeRTOS
#include <string.h>
#include "stddef.h"
#include "FreeRTOS.h"
#include "task.h"

//}includes

void initPort(GPIO_TypeDef* GPIOx, u32_t Pin, GPIOMode_TypeDef Type);
void initPort2(GPIO_TypeDef* GPIOx, u32_t Pin, GPIOMode_TypeDef Type, GPIOSpeed_TypeDef Speed);
u16_t* arraySet(u16_t* Array, u16_t Value, u32_t Amount);
bool Assert(volatile bool Condition, volatile ErrorCode_e ErrorCode);
bool checkSwitch_LowActive(GPIO_TypeDef* GPIOx, u32_t Pin);
bool circa(u32_t A, u32_t B, u8_t Distance);
u16_t toggleValue(u16_t Value, u16_t Max, u16_t Min);

// limits given 
s16_t sLimit(s16_t Value, s16_t Min, s16_t Max);
u16_t uLimit(u16_t Value, u16_t Min, u16_t Max);

// malloc replacement; uses Assert() for out-of-memory detection
void* myMalloc(size_t Size);

// get/ set/ clear port bit
bool portGet(GPIO_TypeDef* GPIOx, u32_t Pin);
void portSet(GPIO_TypeDef* GPIOx, u32_t Pin);
void portClr(GPIO_TypeDef* GPIOx, u32_t Pin);

// delay functions
void programCalibrateUSleep();
void mSleep(u32_t Delay);
void uSleep(u32_t Delay);


#endif
