/*{ ExampleThreading::.c ************************************************
 
 Example code demonstrating how to use inter thread communication.
 
}*/

#include "example_threading.h"

//{ Function definitions *************************************************

void threading_init() {
    leds_initPorts();
}
void threading_start() {
    threading_init();

    et_ThreadInfo_t ThreadInfo;
    ThreadInfo.QueueHandle = xQueueCreate(2, 1);  // create queue for 10x1 bytes

    Assert(NULL != ThreadInfo.QueueHandle, ec_Malloc);

    xTaskCreate(taskProducer,                          // function to start as thread
                (const signed char*) "taskProducer",   // thread name (just for debugging)
                (size_t) 4 * configMINIMAL_STACK_SIZE,     // stack size
                (void *) &ThreadInfo,                  // passed as argument to taskBrightness()
                (unsigned portBASE_TYPE) 1,            // task priority (higher values mean more process time)
                (xTaskHandle*) NULL                    // can return a handle to created task
                );

    xTaskCreate(taskConsumer,                          // function to start as thread
                (const signed char*) "taskConsumer",   // thread name (just for debugging)
                (size_t) 4 * configMINIMAL_STACK_SIZE,     // stack size
                (void *) &ThreadInfo,                  // passed as argument to taskBrightness()
                (unsigned portBASE_TYPE) 1,            // task priority (higher values mean more process time)
                (xTaskHandle*) NULL                    // can return a handle to created task
                );
}
void taskProducer(void *pvParameters) {
    et_ThreadInfo_t* ThreadInfo = (et_ThreadInfo_t*) pvParameters;
    Assert(NULL != ThreadInfo, ec_NULL);
    Assert(NULL != ThreadInfo->QueueHandle, ec_et_InvalidQueueGiven);

    ConfigLED_t ConfigLED = { LED_STAT1, 100, 0 };
    if (0) xTaskCreate(taskControlLed,                          // function to start as thread
                       (const signed char*) "taskControlLed1",   // thread name (just for debugging)
                       (size_t) 2*configMINIMAL_STACK_SIZE, // stack size
                       (void *) &ConfigLED,               // passed as argument to taskControlLed()
                       (unsigned portBASE_TYPE) 1,        // task priority (higher values mean more process time)
                       (xTaskHandle*) NULL                // can return a handle to created task
                       );

    u8_t Brightness = 0; //(u16_t) rand() & 0xfff;
    while (1) {
        // while (xQueueSend(ThreadInfo->QueueHandle, &Brightness, (portTickType) -1) != pdTRUE);

        ConfigLED.OnPeriod   = Brightness;
        ConfigLED.OffPeriod = 100 - Brightness;
        mSleep(900);

        Brightness++;
        if (Brightness == 100) Brightness = 0;
    }
}
void taskConsumer(void *pvParameters) {
    et_ThreadInfo_t* ThreadInfo = (et_ThreadInfo_t*) pvParameters;
    Assert(NULL != ThreadInfo, ec_NULL);
    Assert(NULL != ThreadInfo->QueueHandle, ec_et_InvalidQueueGiven);

    ConfigLED_t ConfigLED = { LED_STAT2, 100, 0 };
    if (0) xTaskCreate(taskControlLed,                          // function to start as thread
                       (const signed char*) "taskControlLed2",   // thread name (just for debugging)
                       (size_t) 2*configMINIMAL_STACK_SIZE, // stack size
                       (void *) &ConfigLED,               // passed as argument to taskControlLed()
                       (unsigned portBASE_TYPE) 1,        // task priority (higher values mean more process time)
                       (xTaskHandle*) NULL                // can return a handle to created task
                       );

    u8_t MyBrightness = 0;
    while (1) {

        if ( xQueueReceive(ThreadInfo->QueueHandle, &MyBrightness,  (portTickType) -1) ) {
            if ( Assert( MyBrightness <= 100, ec_InvalidArgument ) ) {
                ConfigLED.OnPeriod   = MyBrightness;
                ConfigLED.OffPeriod = 100 - MyBrightness;
                mSleep(1000);
            }
        }
    }
}

//} Function definitions
