#!/bin/bash

#
#  Meta install script for STM32-toolchain.
#  written by Gregor Rebel 2011-04-11.
#  
#  This script will automatically run all install_* scripts in current folder.
# 
#  Feel free do distribute and change to your own needs!
#
# Tested successfully on:
# Ubuntu 10.04 Netbook Edition x86
# KUbuntu 11.04 x86
# openSuSE 11.4 x86
# XUbuntu 11.10 x64

Script="$1"

source scripts/installFuncs.sh

if [ "$Script" == "" ]; then
  Files=`ls install_*`

  # prepare for installation
  rm 2>/dev/null ErrorOccured AllInstallsOK
  echo "" >cleanup.sh
  if [ ! -d Logs ]; then
    mkdir Logs
  fi
  for Script in $Files; do
    if [ ! -e Logs/$Script.log ]; then
      echo "" >Logs/$Script.log
    fi
  done
  
  # createLinks
  cat <<END_OF_LINKS >scripts/createLinks.sh
#!/bin/bash
# createLinks.sh created `date`
#
# This script is part of ToolChain_STM32

if [ ! -d extensions.active ]; then
  mkdir extensions.active
fi
cd extensions.active
if [ ! -e readme.txt ]; then
  echo <<"END_OF_README" >readme.txt

                               ToolChain_STM32

                         Extension to central makefile

                       written by Gregor Rebel 2010-2011


All makefile.* files in this folder are automatically being included by ../makefile

The idea of this folder is to activate certain extensions by creating symbolic
links to files inside ../extensions

Example:

cd extensions.active
rm makefile.*
ln -s ../extensions/makefile.free_rtos    .
ln -s ../extensions/makefile.example_leds .
cd ..
./compile.sh

END_OF_README
fi

if [ ! -e makefile.empty ]; then
  echo "" >makefile.empty
  ln -s ../extensions/makefile.free_rtos    .
  ln -s ../extensions/makefile.example_leds .
  ln -s ../extensions/makefile.debug_registers .
fi
cd ..

rm 2>/dev/null extensions;        ln -sv \$Source/ToolChain_STM32/ToolChain/extensions/                                               extensions

cd additionals

END_OF_LINKS

  # run all install scripts
  echo "Runing all install scripts."
  echo "You may check progress by inspecting log files via"
  echo "  tail -f `pwd`/Logs/*"
  echo ""
  for Script in $Files; do
    if [ ! -e ErrorOccured ]; then
      Error=""
      $0 $Script || Error="1"
      if [ "$Error" != "" ]; then
        echo "ERROR occured during $Script !"
        exit 6
      fi
    fi
  done
  
  cp ../Version scripts/
  echo 'cd ..; echo "ToolChain_STM32 v`cat _/Version`"' >>scripts/createLinks.sh

  # remove temporary files
  chmod +x cleanup.sh
  ./cleanup.sh
  
  # installation finished successfully
  touch AllInstallsOK
  addLine $HOME/.bashrc "source `pwd`/scripts/SourceMe.sh"
  addLine scripts/SourceMe.sh "alias commit='git pull && git commit'"
  echo ""
  echo "All tools have been installed successfully."
  echo "Note: Your .bashrc has been updated."
  echo "      Start a new shell to use the new environment!"
  echo ""
else
  echo -n "running `pwd`/$Script.."
  dir Logs
  echo "$Script started @`date`" >Logs/$Script.log
  ./$Script >Logs/$Script.log 2>Logs/$Script.err && OK="1"
  if [ "$OK" == "1" ]; then
    echo "OK"
  else
    echo "ERROR"
    echo ""
    echo "Check logfiles:"
    find `pwd`/Logs/ -name $Script.* -exec echo "less {}" \;
    touch ErrorOccured
    exit 5
  fi
fi
