#!/bin/bash
#
#  Install script for FreeRTOS Multitasking Schedulder for Toolchain_STM32
#  Script written by Gregor Rebel 2010-2011.
# 
#  Feel free do distribute and change to your own needs!
#
# Tested successfully on:
# Ubuntu 10.04 Netbook Edition x86
# KUbuntu 11.04 x86
# openSuSE 11.4 x86
# XUbuntu 11.10 x64

source scripts/installFuncs.sh

if [ "$USER" == "root" ]; then
  echo "$0: ERROR this script should not been run as user root!"
  exit 10
fi

INSTALLPATH="free_rtos"
dir $INSTALLPATH
cd $INSTALLPATH

#{ IMPLEMENT YOUR INSTALL SCRIPT BELOW -----------------------------------------------

if [ ! -e OK.Install ]; then #{

  ERROR=""
  
  if [ ! -e OK.Packages ]; then #{
    installPackage libgmp3-dev 
    installPackage libmpfr-dev
    touch OK.Packages
  fi #}
  
  #Version="6.1.1"
  Version="7.0.2"
  Folder="FreeRTOSv${Version}"
  Archive="FreeRTOSv${Version}.zip"
  if [ ! -d $Folder ]; then
    ERROR=""
    unzip -t $Folder.zip || ERROR="1"
    if [ $ERROR == "1" ]; then
      rm 2>/dev/null $Folder.zip
      echo "Downloading FreeRTOS (server sometimes is very slow).."
      get http://dfn.dl.sourceforge.net/project/freertos/FreeRTOS/V${Version}/ $Archive
    fi
    ERROR=""
    unzip -t $Archive || ERROR="1"
    if [ "$ERROR" == "" ]; then
      unZIP FreeRTOS $Archive
      mv FreeRTOS $Folder
      ln -s $Folder current
      #add2CleanUp ../cleanup.sh "rm -Rf $INSTALLPATH/$Folder"
    else
      echo "ERROR: Zip Archive is corrupted ($Folder.zip)!"
      exit 11
    fi
  fi
  
  File="current/Demo/Common/drivers/ST/STM32F10xFWLib/inc/stm32f10x_systick.h"
  if [ ! -e "${File}_orig" ]; then #{
      makeOrig ${File}
      echo "modifing ${File}.."
      ls -lh $File
      replaceInFile ${File} '#include "stm32f10x_map.h"' 'ToolChain_STM32 ERROR: Do not include this file! #include "stm32f10x_map.h" '
  fi #}
  
  File="current/Demo/Common/drivers/ST/STM32F10xFWLib/inc/stm32f10x_rcc.h"
  if [ ! -e "${File}_orig" ]; then #{
      makeOrig ${File}
      echo "modifing ${File}.."
      ls -lh $File
      replaceInFile ${File} '#include "stm32f10x_map.h"' '//ToolChain_STM32 ERROR: Do not include this file! #include "stm32f10x_map.h" '
  fi #}
  
  File="current/Source/include/mpu_wrappers.h"
  if [ ! -e "${File}_orig" ]; then #{
      mv ${File} ${File}_orig
      echo "modifing ${File}.."
      sed -e "s/\#define PRIVILEGED_FUNCTION __attribute__\(\(section\(\"privileged_functions\"\)\)\)/\#define PRIVILEGED_FUNCTION \/\/__attribute__\(\(section\(\"privileged_functions\"\)\)\)/g" ${File}_orig >${File}
  fi #}
  
  #X File="current/Demo/Common/drivers/ST/STM32F10xFWLib/inc/stm32f10x_type.h"
  #X if [ ! -e "${File}_orig" ]; then #{
  #X     makeOrig ${File}
  #X     echo "`pwd`> modifing ${File}.."
  #X     Comment="// previous content of this file is already defined in std_peripherals_library\/CMSIS\...\/stm32f10x.h"
  #X     echo "$Comment" >${File}
  #X 
  #X     #X replaceInFile ${File} "typedef signed long  s32;" "// typedef signed long  s32; // $Comment"
  #X     #X replaceInFile ${File} "typedef signed short s16;" "// typedef signed short s16; // $Comment"
  #X     #X replaceInFile ${File} "typedef signed char  s8;"  "// typedef signed char  s8;  // $Comment"
  #X   
  #X     #X replaceInFile ${File} "typedef volatile signed long  vs32;" "// typedef volatile signed long  vs32; // $Comment"
  #X     #X replaceInFile ${File} "typedef volatile signed short vs16;" "// typedef volatile signed short vs16; // $Comment"
  #X     #X replaceInFile ${File} "typedef volatile signed char  vs8;"  "// typedef volatile signed char  vs8; // $Comment"
  #X 
  #X     #X replaceInFile ${File} "typedef unsigned long  u32;"         "// typedef unsigned long  u32; // $Comment"
  #X     #X replaceInFile ${File} "typedef unsigned short u16;"         "// typedef unsigned short u16; // $Comment"
  #X     #X replaceInFile ${File} "typedef unsigned char  u8;"          "// typedef unsigned char  u8; // $Comment"
  #X fi #}
  
  Comment="correct type of usStackDepth improves visibility during gcc debugging"
  File="current/Source/include/task.h"
  if [ ! -e "${File}_orig" ]; then #{
      makeOrig ${File}
      echo "`pwd`> modifing ${File}.. ($Comment)"
  
      replaceInFile ${File} "unsigned short usStackDepth;" "size_t usStackDepth; //ToolChain_STM32 $Comment"
  fi #}
  
  File="current/Source/tasks.c"
  if [ ! -e "${File}_orig" ]; then #{
      makeOrig ${File}
      echo "`pwd`> modifing ${File}.. ($Comment)"
  
      replaceInFile ${File} "unsigned short usStackDepth;" "size_t usStackDepth; //ToolChain_STM32 $Comment"
  fi #}

  if [ -d $Folder ]; then
    echo "" >OK.Install
  fi
fi #} [ ! -e OK.Install ]; then
if [   -e OK.Install ]; then #{
  cat <<END_OF_MAKEFILE >../extensions/makefile.$INSTALLPATH #{
#
#                               ToolChain_STM32
#
#                         Extension to central makefile
#
#                   makefile written by Gregor Rebel 2010-2011
#

EXTENSION_$INSTALLPATH = 1

FREE_RTOS = 1

ifdef FREE_RTOS

#activate FreeRTOS parts of source code
COMPILE_OPTS += -DFREE_RTOS -DEXTENSION_$INSTALLPATH

# add include directories
INCLUDE_DIRS += -I additionals/free_rtos/Source/include/ \\
                -I additionals/free_rtos/Source/portable/GCC/ARM_CM3/ \\

#                -I additionals/free_rtos/Demo/Common/drivers/ST/STM32F10xFWLib/inc/ \

# add source directories
VPATH +=  additionals/free_rtos/Source/ \\
          additionals/free_rtos/Source/portable/MemMang/ \\
          additionals/free_rtos/Source/portable/GCC/ARM_CM3/ \\

# disable warnings 
# (W_CONVERSION = -Wconversion produces lots of warnings from FreeRTOS code)
W_CONVERSION = 

# redefine FreeRTOS exception handlers --------------------------------------
# in additionals/free_rtos/Source/portable/GCC/ARM_CM3/port.c to match with CMSIS vector handlers 
# in std_peripherals_library/CMSIS/CM3/DeviceSupport/ST/STM32F10x/startup/gcc_ride7/startup_stm32f10x_md.s
# (see http://www.mikrocontroller.net/topic/148988)
AS_OPTS = -DUSART1_IRQHandler=vUARTInterruptHandler \\
          -DADC1_2_IRQHandler=ADC_IRQHandler \\
          -DSysTick_Handler=xPortSysTickHandler \\
          -DSVC_Handler=vPortSVCHandler \\
          -DPendSV_Handler=xPortPendSVHandler

# add FreeRTOS object-files
MAIN_OBJS += port.o queue.o list.o tasks.o stm32_io.o
#? syscalls.o 

# activate one of the supplied memory allocators to use
MAIN_OBJS += heap_1.o # can only allocate    (deterministic, fast, minimal memory usage)
#MAIN_OBJS += heap_2.o # allocate/ deallocate (deterministic, high memory usage)
#MAIN_OBJS += heap_3.o # allocate/ deallocate (full implementation uses malloc()/ free())

endif

END_OF_MAKEFILE
#}

  addLine ../scripts/createLinks.sh 'rm 2>/dev/null free_rtos;              ln -sv $Source/ToolChain_STM32/ToolChain/free_rtos/current                                          free_rtos'
  echo "$INSTALLPATH installed successfully"
#}
else
  echo "failed to install $INSTALLPATH"
  exit 10
fi

#} IMPLEMENT YOUR INSTALL SCRIPT ABOVE -----------------------------------------------

exit 0
