#!/bin/bash
#
#  Install script for ST Standard Peripherals Library
#  Script written by Gregor Rebel 2010-2011.
# 
#  Feel free do distribute and change to your own needs!
#
# Tested successfully on:
# Ubuntu 10.04 Netbook Edition x86
# KUbuntu 11.04 x86
# openSuSE 11.4 x86
# XUbuntu 11.10 x64

source scripts/installFuncs.sh

if [ "$USER" == "root" ]; then
  echo "$0: ERROR this script should not been run as user root!"
  exit 10
fi

INSTALLPATH="std_peripherals_library"
dir $INSTALLPATH
cd $INSTALLPATH

#{ IMPLEMENT YOUR INSTALL SCRIPT BELOW -----------------------------------------------

if [ ! -e OK.Install ]; then #{

  if [ ! -e OK.Packages ]; then #{
    installPackage libgmp3-dev
    installPackage libmpfr-dev
    installPackage xchm
    installPackage chmlib
    installPackage chmsee
    installPackage kchmviewer
    touch OK.Packages
  fi #}
  
  Archive="stm32f10x_stdperiph_lib.zip"
  # get http://www.st.com/stonline/products/support/micro/files/ $Archive
  get http://www.st.com/internet/com/SOFTWARE_RESOURCES/SW_COMPONENT/FIRMWARE/ $Archive
  
  # find name ofu only directory
  # std_peripherals_library/STM32F10x_StdPeriph_Lib_V3.4.0/Libraries/CMSIS/CM3/DeviceSupport/ST/STM32F10x/stm32f10x.h
  
  unZIP STM32F10x_StdPeriph_Lib $Archive || mv -f $Archive ${Archive}_bad
  rm current 2>/dev/null
  LibraryDir=`find ./  -maxdepth 1 -mindepth 1 -type d -print`
  ln -sv $LibraryDir current
  cd current
  mv *.chm ../../../Documentation/
  
  File="Libraries/CMSIS/CM3/DeviceSupport/ST/STM32F10x/stm32f10x.h"
  if [ -e ${File}_orig ]; then
      mv -f ${File}_orig $File 
  fi
  mv -f $File ${File}_orig
  
  echo "  creating macro assert_param in $File"
  Note="  Note: Manually remove assert_param from your stm32f10x_conf.h !"
  echo $Note
  addLine ../Notes.txt $Note
  addLine ../Notes.txt "StdPeripheralLibrary-> http://www.mikrocontroller.net/articles/STM32F10x_Standard_Peripherals_Library  (german) folder ToolChain/documentation"
  
  DestFolder="Libraries/CMSIS/CM3/DeviceSupport/ST/STM32F10x"
  if [ ! -e $DestFolder/system_stm32f10x.c ]; then
    # system_stm32f10x.c is required to be found by the compiler so we put it aside its header file
    cp -v Project/STM32F10x_StdPeriph_Template/system_stm32f10x.c $DestFolder
    addLine ../Notes.txt "system_stm32f10x.c moved beside its header file (-> https://my.st.com/public/STe2ecommunities/mcu/Lists/ARM%20CortexM3%20STM32/Flat.aspx?RootFolder=%2Fpublic%2FSTe2ecommunities%2Fmcu%2FLists%2FARM%20CortexM3%20STM32%2FMissing%20file%20in%20new%203.4.0%20libraries%20system_stm32f10x.c&FolderCTID=0x01200200770978C69A1141439FE559EB459D758000626BE2B829C32145B9EB5739142DC17E&currentviews=320)"
  fi

  cat >script.sed <<"END_OF_TEXT"
/#include <stdint.h>/a\
#ifdef  USE_FULL_ASSERT\n
\n
// This assert_param macro has been copied here from stm32f10x_conf.h to\n
// circumvent gcc compiler errors.\n
\n
/**\n
  * @brief  The assert_param macro is used for function's parameters check.\n
  * @param  expr: If expr is false, it calls assert_failed function\n
  *   which reports the name of the source file and the source\n
  *   line number of the call that failed.\n 
  *   If expr is true, it returns no value.\n
  * @retval None\n
  */\n
  #define assert_param(expr) ((expr) ? (void)0 : assert_failed((uint8_t *)__FILE__, __LINE__))\n
  void assert_failed(uint8_t* file, uint32_t line);\n
#else\n
  #define assert_param(expr) ((void)0)\n
#endif /* USE_FULL_ASSERT */\n

END_OF_TEXT

  # concatenate all lines which contain \n (sed requires this)
  awk '/\\n/{printf "%s",$0;next}{print}' script.sed >script2.sed
  
  # insert lines after "#include <stdint.h>"
  sed -f script2.sed ${File}_orig >$File

  cd ..

  echo "" >OK.Install

fi #} [ ! -e OK.Install ]; then
if [   -e OK.Install ]; then #{
  cat <<END_OF_MAKEFILE >../extensions/makefile.$INSTALLPATH #{
#
#                               ToolChain_STM32
#
#                         Extension to central makefile
#
#                   makefile written by Gregor Rebel 2010-2011
#

EXTENSION_$INSTALLPATH = 1


END_OF_MAKEFILE
#}

  addLine ../scripts/createLinks.sh 'rm 2>/dev/null std_peripherals_library; ln -sv $Source/ToolChain_STM32/ToolChain/std_peripherals_library/current/Libraries/                  std_peripherals_library'
  echo "Installed successfully: $INSTALLPATH"
#}
else
  echo "failed to install $INSTALLPATH"
  exit 10
fi

#} IMPLEMENT YOUR INSTALL SCRIPT ABOVE -----------------------------------------------

exit 0
