#!/bin/bash

#
#  Install script for USB serial port chip FTDI
#  Script written by Gregor Rebel 2010-2011.
# 
#  Feel free do distribute and change to your own needs!
#
# Tested successfully on:
# Ubuntu 10.04 Netbook Edition x86
# KUbuntu 11.04 x86
# openSuSE 11.4 x86
# XUbuntu 11.10 x64

source scripts/installFuncs.sh


if [ "$USER" == "root" ]; then
  echo "$0: ERROR this script should not been run as user root!"
  exit 10
fi

INSTALLPATH="libftd2xx"
dir $INSTALLPATH
cd $INSTALLPATH

#{ IMPLEMENT YOUR INSTALL SCRIPT BELOW -----------------------------------------------

if [ ! -e OK.Install ]; then #{
  
  echo "installing in $INSTALLPATH ..."
  
  OldFolder=`pwd`
  if [ ! -e OK.Packages ]; then #{
    installPackage libusb-1.0-0-dev 
    installPackage libusb-dev
    touch OK.Packages
  fi #}
  
  Version="1.0.4"
  Folder="libftd2xx${Version}"
  get http://www.ftdichip.com/Drivers/D2XX/Linux/ $Folder.tar.gz
  untgz $Folder $Folder.tar.gz
  add2CleanUp ../cleanup.sh "rm -Rf $INSTALLPATH/$Folder"
  cd $Folder
  
  findPrefix
  
  sudo cp -v ftd2xx.h $Prefix/include
  sudo cp -v WinTypes.h $Prefix/include
  sudo chmod a+r $Prefix/include/ftd2xx.h $Prefix/include/WinTypes.h
  
  ArchitectureIs32Bit=`uname -a | grep i686`
  if [ "$ArchitectureIs32Bit" != "" ]; then
    echo "installing for 32 bit architecture.."
  
    sudo cp -v build/i386/libftd2xx.so.${Version} $Prefix/lib/
    cd $Prefix/lib
    createLink libftd2xx.so.${Version} libftd2xx.so root
    cd /usr/lib
    createLink $Prefix/lib/libftd2xx.so.${Version} libftd2xx.so root
  else  # 64 Bit OS
    echo "installing for 64 bit architecture.."
  
    sudo cp -v build/x86_64/libftd2xx.so.${Version} $Prefix/lib64/
    cd $Prefix/lib64
    createLink libftd2xx.so.${Version} libftd2xx.so root
    cd /usr/lib
    createLink $Prefix/lib/libftd2xx.so.${Version} libftd2xx.so root
  fi
  sudo ldconfig
  
  cd "$OldFolder"

  echo "" >OK.Install

fi #} [ ! -e OK.Install ]; then
if [   -e OK.Install ]; then #{
cat <<END_OF_MAKEFILE >../extensions/makefile.$INSTALLPATH #{  
#
#                               ToolChain_STM32
#
#                         Extension to central makefile
#
#                   makefile written by Gregor Rebel 2010-2011
#

EXTENSION_$INSTALLPATH = 1

END_OF_MAKEFILE
#}

  echo "Installed successfully: $INSTALLPATH"
#}
else
  echo "failed to install $INSTALLPATH"
  exit 10
fi

#} IMPLEMENT YOUR INSTALL SCRIPT ABOVE -----------------------------------------------

exit 0
