#!/bin/bash

#
#  Install script for USB serial port chip driver as required to compile OpenOCD
#  Script written by Gregor Rebel 2010-2011.
#  libFTDI - FTDI USB driver with bitbang mode
# 
#  Feel free do distribute and change to your own needs!
#
# Tested successfully on:
# Ubuntu 10.04 Netbook Edition x86
# KUbuntu 11.04 x86
# openSuSE 11.4 x86
# XUbuntu 11.10 x64

source scripts/installFuncs.sh


if [ "$USER" == "root" ]; then
  echo "$0: ERROR this script should not been run as user root!"
  exit 10
fi

INSTALLPATH="libftdi"
dir $INSTALLPATH
cd $INSTALLPATH

#{ IMPLEMENT YOUR INSTALL SCRIPT BELOW -----------------------------------------------

if [ ! -e OK.Install ]; then #{

  echo "installing in $INSTALLPATH ..."
  
  #Version="0.18"
  Version="0.19"
  
  if [ ! -e OK.Packages ]; then #{
    installPackage "libusb-devel"
    installPackage "libusb-dev"
    installPackage "libusb-1_0-devel"
    installPackage "libusb*-dev*"
    installPackage "libusb-compat-devel"
    installPackage "libusb"
    installPackage "libusb-dev"
    touch OK.Packages
  fi #}
  
  Folder="libftdi-${Version}"
  get http://www.intra2net.com/de/produkte/opensource/ftdi/TGZ/ $Folder.tar.gz
  untgz $Folder $Folder.tar.gz
  add2CleanUp ../cleanup.sh "rm -Rf $INSTALLPATH/$Folder"
  
  InstallInHomeDir="0"
  if [ "$InstallInHomeDir" == "1" ]; then
    Prefix=`pwd`  # install into subfolder of ToolChain/ for current user only
  else
    findPrefix   # install for all users
  fi
  cd $Folder
  ./configure --prefix=$Prefix   >configure.log 2>configure.err && \
              make -j2           >make.log      2>make.err && \
              sudo make install  >install.log   2>install.err && \
              OK="1"
  cd ..
  
  if [ "$OK" == "1" ]; then
    if [ "$InstallInHomeDir" != "1" ]; then
      sudo ldconfig
    fi
    cp -R $Folder/doc ../../Documentation/libftdi

    echo "" >OK.Install
  else
    echo "Error occured during compilation!"
    echo "check logfiles:"
    find `pwd`/ -name "*.log" -exec echo "  less {}" \;
    find `pwd`/ -name "*.err" -exec echo "  less {}" \;
  fi

fi #} [ ! -e OK.Install ]; then
if [   -e OK.Install ]; then #{
  if [ "$InstallInHomeDir" == "1" ]; then
    cat <<END_OF_MAKEFILE >../extensions/makefile.$INSTALLPATH #{
#
#                               ToolChain_STM32
#
#                         Extension to central makefile
#
#                   makefile written by Gregor Rebel 2010-2011
#

EXTENSION_$INSTALLPATH = 1

END_OF_MAKEFILE
#}

    addLine ../scripts/SourceMe.sh "export LD_LIBRARY_PATH=\"$Prefix/lib:\$LD_LIBRARY_PATH\""
    addLine ../scripts/SourceMe.sh "export PATH=\"$Prefix/bin:\$PATH\""
  fi
  echo "Installed successfully: $INSTALLPATH"
#}
else
  echo "failed to install $INSTALLPATH"
  exit 10
fi

#} IMPLEMENT YOUR INSTALL SCRIPT ABOVE -----------------------------------------------

exit 0
