#!/bin/bash

#
#  Install script for Open On-Chip Debugger tool made by Dominic Rath.
#  Script written by Gregor Rebel 2010-2011.
#  OpenOCD - On Chip Debugger
# 
#  Feel free do distribute and change to your own needs!
#
# Tested successfully on:
# Ubuntu 10.04 Netbook Edition x86
# KUbuntu 11.04 x86
# openSuSE 11.4 x86
# XUbuntu 11.10 x64

source scripts/installFuncs.sh

if [ "$USER" == "root" ]; then
  echo "$0: ERROR this script should not been run as user root!"
  exit 10
fi

INSTALLPATH="open_ocd"
dir $INSTALLPATH
cd $INSTALLPATH

#{ IMPLEMENT YOUR INSTALL SCRIPT BELOW -----------------------------------------------

if [ ! -e OK.Install ]; then #{

echo "installing in $INSTALLPATH ..."

InstallDir=`pwd`
if [ ! -e OK.Packages ]; then #{
  installPackage texlive
  touch OK.Packages
fi #}
  
InstallInHomeDir="1"
if [ "$InstallInHomeDir" == "1" ]; then
  Prefix=`pwd`  # install into subfolder of ToolChain/ for current user only
else
  findPrefix    # install for all users
fi

#File="openocd-0.4.0"
File="openocd-0.5.0"
get http://download.berlios.de/openocd/ ${File}.tar.bz2
get http://download.berlios.de/openocd/ openocd.pdf "OpenOCD_Manual.pdf" ../../Documentation/

OldDir=`pwd`
cd ../../Documentation
mkdir OpenOCD
cd OpenOCD/
wget -rnp -nd http://openocd.berlios.de/doc/html/index.html
cd $OldDir

untgj ${File} ${File}.tar.bz2
add2CleanUp ../cleanup.sh "rm -Rf $INSTALLPATH/$File"

if [ ! -d ${File} ]; then
  echo "ERROR: Cannot download file (folder ${File} not found in `pwd`)!"
  exit 5
fi
cd ${File}/
  echo "compiling $File ----------------------------------------------"
  echo "clean..." && make clean >clean.log 2>&1

  #./configure  --prefix=$Prefix --enable-usbprog --enable-ioutil --enable-oocd_trace --enable-jlink --enable-ft2232_ftd2xx && make -j2 && sudo make install && touch Install_OK
  IncludeDir="`pwd`/../../libftdi/"
  OK=""
  echo "configure >configure.log..." && ./configure --prefix=$Prefix \
                                                    --enable-ioutil --enable-oocd_trace --enable-jlink \
                                                    --enable-ft2232_libftdi \
                                                    >configure.log 2>&1  && OK="1"
  if [ "$OK" == "" ]; then
    echo "Error during configuring ${File}!"
    exit 10
  fi
  for Makefile in $( find ./ -name Makefile ); do
    echo "replacing Werror in $Makefile .."
    replaceInFile $Makefile "-Werror" " "
  done
  if [ "$AmountCPUs" == "" ]; then
    AmountCPUs="1"
  fi
  AmountCPUs=$(( $AmountCPUs * 2 ))
  echo "make      >make.log..."      && make -j$AmountCPUs >make.log 2>&1 && \
  echo "pdf       >pdf.log..."       && make pdf -j$AmountCPUs >pdf.log 2>&1 ; \
  echo "install   >install.log..."   && sudo make install >install.log 2>&1 && \
  touch Install_OK

  if [ ! -e Install_OK ]; then
    echo "Error during compiling ${File}!"
    echo "Check log files:"
    find `pwd`/ -name "*.log" -exec echo "  less {}" \;
    find `pwd`/ -name "*.err" -exec echo "  less {}" \;
    exit 10
  fi
  sudo chown -R $USER: .

cd ..

if [ -e "$Prefix/bin/openocd" ]; then
    IsInstalled=`$Prefix/bin/openocd --version  2>&1 | grep Debugger`
    if [ "$IsInstalled" != "" ]; then
      sudo chmod 4755 $Prefix/bin/openocd
      echo "Installed version: $IsInstalled"
      addLine ../../cleanup.sh "rm -Rf `pwd`/${File}/"
      cd $InstallDir
      
      echo "" >OK.Install
    fi
fi
fi #} [ ! -e OK.Install ]; then
if [  -e OK.Install ]; then #{
  cat <<END_OF_MAKEFILE >../extensions/makefile.$INSTALLPATH #{
#
#                               ToolChain_STM32
#
#                         Extension to central makefile
#
#                   makefile written by Gregor Rebel 2010-2011
#

EXTENSION_$INSTALLPATH = 1

END_OF_MAKEFILE
#}

  addLine ../scripts/SourceMe.sh "export PATH=\"$Prefix/bin/:\$PATH\""
  echo "Installed successfully: $INSTALLPATH"
#}
else
  echo "failed to install $INSTALLPATH"
  exit 10
fi

cd ..
#} IMPLEMENT YOUR INSTALL SCRIPT ABOVE -----------------------------------------------

exit 0
