#!/usr/bin/perl -w

use strict;
use File::Spec::Functions qw(rel2abs);
use File::Basename;
my $ToolChainDir = dirname(rel2abs($0));

my $ProjectName = shift(@ARGV);
my $AuthorName  = shift(@ARGV);

unless ($AuthorName) {
  print "$0 PROJECT_NAME AUTHOR_NAME\n";
  print "Creates a new copy of Template project. Automatically sets project name and author in all files.\n";
  print "\n";
  exit 5;
}


unless ( -d "$ToolChainDir/Template" ) {
  print STDERR "$0 ERROR: Cannot find folder Template in $ToolChainDir/ !\n";
  exit 10;
}
if (-d $ProjectName) {
  print STDERR "$0 ERROR: Directory $ProjectName already exists in `pwd` !\n";
  exit 11;
}
if (-e $ProjectName) {
  print STDERR "$0 ERROR: File $ProjectName is in the way in `pwd` !\n";
  exit 12;
}

print "Creating new project $ProjectName for $AuthorName in $ToolChainDir/..\n";
system("cp -R '$ToolChainDir/Template' $ProjectName");
unless (-d $ProjectName) {
  print STDERR "$0 ERROR: Cannot create directory $ProjectName in `pwd` !\n";
  exit 13;
}
chdir($ProjectName);
system("./_/clean.sh");
foreach my $FileName (qw( TemplateName.config  TemplateName.creator  TemplateName.creator.user  TemplateName.files  TemplateName.includes ) ) {
  my $NewName = $FileName;
  $NewName =~ s/TemplateName/$ProjectName/;
  print "$FileName -> $NewName\n";
  rename($FileName, $NewName);
}

print "searching for files to be processed...\n";
my @Files2Process;
foreach my $Keyword ( qw(TemplateName TemplateAuthor) ) {
  my $FilesFound = `grep $Keyword *`;
  my @FilesFound = split("\n", $FilesFound);
  foreach my $FileFound (@FilesFound) {
    my @Parts = split(':', $FileFound);
    my $FileName = $Parts[0];
    if ($FileName) {
      push @Files2Process, $FileName;
    }
  }
}

my $Error = '';
foreach my $File2Process (@Files2Process) {
  my $OldError = $Error;
  open(IN, "<$File2Process") or $Error .= "Cannot read from file '$File2Process' ($!)\n";
  my @Lines;
  if ($OldError eq $Error) {
    @Lines = <IN>;
    close(IN);
    print "modifying $File2Process..\n";
    foreach my $Line (@Lines) {
      $Line =~ s/TemplateName/$ProjectName/g;
      $Line =~ s/TemplateAuthor/$AuthorName/g;
    }
    $OldError = $Error;
    open(OUT, ">$File2Process") or $Error .= "Cannot write to file '$File2Process' ($!)\n";
    if ($OldError eq $Error) {
      print OUT join("", @Lines);
      close(OUT);
    }
  }
  if ($Error) { print $Error; }
}


print "\n";
print "Project $ProjectName has been created successfully.\n";
print "Now start QtCreator and open `pwd`/$ProjectName.creator\n";
print "\n";

exit 0;
